@@ -28,6 +28,8 @@
 #include "importer.h"
 #include "jsapi-util.h"
 #include "profiler.h"
+#include "native.h"
+#include "byteArray.h"
 
 #include <util/log.h>
 #include <util/error.h>
@@ -107,6 +109,8 @@ gjs_log(JSContext *context,
         return JS_FALSE;
     }
 
+    JS_BeginRequest(context);
+
     /* JS_ValueToString might throw, in which we will only
      *log that the value could be converted to string */
     exc_state = JS_SaveExceptionState(context);
@@ -117,15 +121,19 @@ gjs_log(JSContext *context,
 
     if (jstr == NULL) {
         gjs_debug(GJS_DEBUG_LOG, "<cannot convert value to string>");
+        JS_EndRequest(context);
         return JS_TRUE;
     }
 
-    if (!gjs_string_to_utf8(context, STRING_TO_JSVAL(jstr), &s))
+    if (!gjs_string_to_utf8(context, STRING_TO_JSVAL(jstr), &s)) {
+        JS_EndRequest(context);
         return JS_FALSE;
+    }
 
     gjs_debug(GJS_DEBUG_LOG, "%s", s);
     g_free(s);
 
+    JS_EndRequest(context);
     return JS_TRUE;
 }
 
@@ -146,6 +154,8 @@ gjs_log_error(JSContext *context,
         return JS_FALSE;
     }
 
+    JS_BeginRequest(context);
+
     exc = argv[0];
 
     /* JS_ValueToString might throw, in which we will only
@@ -159,16 +169,20 @@ gjs_log_error(JSContext *context,
     if (jstr == NULL) {
         gjs_debug(GJS_DEBUG_ERROR, "<cannot convert value to string>");
         gjs_log_exception_props(context, exc);
+        JS_EndRequest(context);
         return JS_TRUE;
     }
 
-    if (!gjs_string_to_utf8(context, STRING_TO_JSVAL(jstr), &s))
+    if (!gjs_string_to_utf8(context, STRING_TO_JSVAL(jstr), &s)) {
+        JS_EndRequest(context);
         return JS_FALSE;
+    }
 
     gjs_debug(GJS_DEBUG_ERROR, "%s", s);
     gjs_log_exception_props(context, exc);
     g_free(s);
 
+    JS_EndRequest(context);
     return JS_TRUE;
 }
 
@@ -182,6 +196,8 @@ gjs_print_parse_args(JSContext *context,
     gchar *s;
     guint n;
 
+    JS_BeginRequest(context);
+
     str = g_string_new("");
     JS_EnterLocalRootScope(context);
     for (n = 0; n < argc; ++n) {
@@ -201,6 +217,7 @@ gjs_print_parse_args(JSContext *context,
         if (jstr != NULL) {
             if (!gjs_string_to_utf8(context, STRING_TO_JSVAL(jstr), &s)) {
                 JS_LeaveLocalRootScope(context);
+                JS_EndRequest(context);
                 g_string_free(str, TRUE);
                 return JS_FALSE;
             }
@@ -211,6 +228,7 @@ gjs_print_parse_args(JSContext *context,
                 g_string_append_c(str, ' ');
         } else {
             JS_LeaveLocalRootScope(context);
+            JS_EndRequest(context);
             *buffer = g_string_free(str, TRUE);
             if (!*buffer)
                 *buffer = g_strdup("<invalid string>");
@@ -221,6 +239,7 @@ gjs_print_parse_args(JSContext *context,
     JS_LeaveLocalRootScope(context);
     *buffer = g_string_free(str, FALSE);
 
+    JS_EndRequest(context);
     return JS_TRUE;
 }
 
@@ -315,6 +334,8 @@ gjs_context_class_init(GjsContextClass *klass)
     g_object_class_install_property(object_class,
                                     PROP_IS_LOAD_CONTEXT,
                                     pspec);
+
+    gjs_register_native_module("byteArray", gjs_define_byte_array_stuff, 0);
 }
 
 static void
@@ -330,7 +351,9 @@ gjs_context_dispose(GObject *object)
     }
 
     if (js_context->global != NULL) {
+        JS_BeginRequest(js_context->context);
         JS_RemoveRoot(js_context->context, &js_context->global);
+        JS_EndRequest(js_context->context);
         js_context->global = NULL;
     }
 
@@ -386,7 +409,7 @@ gjs_context_finalize(GObject *object)
         g_strfreev(js_context->search_path);
         js_context->search_path = NULL;
     }
-    
+
     g_static_mutex_lock(&contexts_lock);
     all_contexts = g_list_remove(all_contexts, object);
     g_static_mutex_unlock(&contexts_lock);
@@ -537,6 +560,8 @@ gjs_context_constructor (GType                  type,
     if (js_context->context == NULL)
         gjs_fatal("Failed to create javascript context");
 
+    JS_BeginRequest(js_context->context);
+
     /* same as firefox, see discussion at
      * https://bugzilla.mozilla.org/show_bug.cgi?id=420869 */
     JS_SetScriptStackQuota(js_context->context, 100*1024*1024);
@@ -652,6 +677,8 @@ gjs_context_constructor (GType                  type,
         js_context->profiler = gjs_profiler_new(js_context->runtime);
     }
 
+    JS_EndRequest(js_context->context);
+
     g_static_mutex_lock (&contexts_lock);
     all_contexts = g_list_prepend(all_contexts, object);
     g_static_mutex_unlock (&contexts_lock);
@@ -723,11 +750,11 @@ gjs_context_new_with_search_path(char** search_path)
 
 /**
  * gjs_context_get_all:
- * 
+ *
  * Returns a newly-allocated list containing all known instances of #GjsContext.
  * This is useful for operating on the contexts from a process-global situation
  * such as a debugger.
- * 
+ *
  * Return value: (element-type GjsContext) (transfer full): Known #GjsContext instances
  */
 GList*
@@ -805,6 +832,11 @@ gjs_context_eval(GjsContext *js_context,
                   "Exception was set prior to JS_EvaluateScript()");
     }
 
+    /* JS_EvaluateScript requires a request even though it sort of seems like
+     * it means we're always in a request?
+     */
+    JS_BeginRequest(js_context->context);
+
     retval = JSVAL_VOID;
     if (!JS_EvaluateScript(js_context->context,
                            js_context->global,
@@ -869,6 +901,8 @@ gjs_context_eval(GjsContext *js_context,
 
     g_object_unref(G_OBJECT(js_context));
 
+    JS_EndRequest(js_context->context);
+
     return success;
 }
 
